from django.shortcuts import render, get_object_or_404, redirect
from django.http import HttpResponse
from django.views import generic


from .models import Transaction, TransactionForm, Invoice, InvoiceForm


class IndexView(generic.ListView):
    template_name = "main/index.html"
    context_object_name = "transactions"

    def get_queryset(self):
        return Transaction.objects.order_by("-date")[:5]


def transaction(request, uuid=None):
    if uuid is None:
        _transaction = Transaction()
        _invoices = []
    else:
        _transaction = get_object_or_404(Transaction, id=uuid)
        _invoices = Invoice.objects.filter(transaction=_transaction)
    return render(
        request,
        "main/transaction.html",
        {
            "transaction": _transaction,
            "form": TransactionForm(instance=_transaction),
            "invoices": _invoices,
            "invoice_form": InvoiceForm(instance=Invoice(transaction=_transaction)),
        },
    )


def update_transaction(request, uuid):
    try:
        _transaction = Transaction.objects.get(id=uuid)
    except Transaction.DoesNotExist:
        _transaction = Transaction(id=uuid)
    _form = TransactionForm(request.POST, instance=_transaction)
    _form.save()
    return redirect(transaction, uuid=uuid)


def invoice(request, uuid):
    _invoice = get_object_or_404(Invoice, id=uuid)
    with _invoice.file.open() as _file:
        return HttpResponse(_file.read(), content_type="application/pdf")


def add_invoice(request, uuid):
    _transaction = get_object_or_404(Transaction, id=uuid)
    _invoice = Invoice(transaction=_transaction)
    _form = InvoiceForm(request.POST, request.FILES, instance=_invoice)
    _form.save()
    return redirect(transaction, uuid=uuid)


def del_invoice(request, uuid, invoice_id):
    _invoice = get_object_or_404(Invoice, id=invoice_id)
    _invoice.delete()
    return redirect(transaction, uuid=uuid)
